package imt.concours;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Set;

import org.junit.jupiter.api.Test;

import imt.ingredients.Ingredient;
import imt.ingredients.Plat;
import imt.personnes.Chef;
import imt.personnes.Genre;
import imt.personnes.Jury;
import imt.personnes.Specialite;

public class ConcoursTest {
	public static Jury jury1 = new Jury(1, "nom1", "prenom1", "06", Genre.HOMME);
	public static Jury jury2 = new Jury(2, "nom2", "prenom2", "06", Genre.HOMME);
	public static Jury jury3 = new Jury(3, "nom3", "prenom3", "06", Genre.HOMME);

	public static Chef chef1 = new Chef(1, "chef1", "prenomchef1", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(1, "plat1", new Ingredient[] {})));
	public static Chef chef2 = new Chef(2, "chef2", "prenomchef2", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(2, "plat2", new Ingredient[] {})));
	public static Chef chef3 = new Chef(3, "chef3", "prenomchef3", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(3, "plat3", new Ingredient[] {})));
	public static Chef chef4 = new Chef(4, "chef4", "prenomchef4", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(4, "plat4", new Ingredient[] {})));
	public static Chef chef5 = new Chef(5, "chef5", "prenomchef5", "07", Genre.HOMME, 2, Specialite.PATISSIER,
			Set.of(new Plat(5, "plat5", new Ingredient[] {})));

	public static Proposition proposition1 = new Proposition(new Plat(1, "plat1", new Ingredient[] {}), chef1);
	public static Proposition proposition2 = new Proposition(new Plat(2, "plat2", new Ingredient[] {}), chef2);
	public static Proposition proposition3 = new Proposition(new Plat(3, "plat3", new Ingredient[] {}), chef3);
	public static Proposition proposition4 = new Proposition(new Plat(4, "plat4", new Ingredient[] {}), chef4);
	public static Proposition proposition5 = new Proposition(new Plat(5, "plat5", new Ingredient[] {}), chef5);
	public static Set<Proposition> propositions = Set.of(
			proposition1,
			proposition2,
			proposition3,
			proposition4,
			proposition5);

	@Test
	void concours_trois_jurys_cinq_chefs() {
		var propositions = Set.of(
				proposition1,
				proposition2,
				proposition3,
				proposition4);

		assertThrows(IllegalArgumentException.class, () -> new Concours(jury1, jury2, jury3, propositions));
	}

	@Test
	void les_chefs_proposent_un_plat_qu_ils_savent_faire() {
		var propositions = Set.of(
				proposition1,
				proposition2,
				proposition3,
				proposition4,
				new Proposition(new Plat(99, "je sais pas faire donc exception", new Ingredient[]{}), chef5));

		assertThrows(IllegalArgumentException.class, () -> new Concours(jury1, jury2, jury3, propositions));
	}

	@Test
	void fin_du_concours() {
		Concours concours = new Concours(jury1, jury2, jury3, propositions);

		assertEquals(false, concours.estTermine());

		concours.noter(proposition1, 10);
		concours.noter(proposition2, 3);
		concours.noter(proposition3, 8);
		concours.noter(proposition4, 9);
		concours.noter(proposition5, 18);

		assertEquals(true, concours.estTermine());
	}

	@Test
	void classement_actuel() {
		Concours concours1 = new Concours(jury1, jury2, jury3, propositions);

		List<Proposition> expected = new ArrayList<>();
		expected.add(proposition1);
		expected.add(proposition2);
		expected.add(proposition3);
		expected.add(proposition4);
		expected.add(proposition5);

		proposition1.setNote(Optional.of(1));
		proposition2.setNote(Optional.of(2));
		proposition3.setNote(Optional.of(3));
		proposition4.setNote(Optional.of(4));
		// proposition5.setNote(Optional.of(100));

		assertThat(concours1.classement()).isEqualTo(expected);

		System.out.println(concours1.classement());
	}
}
